@@ -1,29 +1,26 @@
 /* Test file for mpfr_set_str.
 
-Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
-Contributed by the Arenaire and Cacao projects, INRIA.
+Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+Contributed by the Arenaire and Caramel projects, INRIA.
 
-This file is part of the MPFR Library.
+This file is part of the GNU MPFR Library.
 
-The MPFR Library is free software; you can redistribute it and/or modify
+The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at your
+the Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
-The MPFR Library is distributed in the hope that it will be useful, but
+The GNU MPFR Library is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
+http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
-#include <limits.h>
 
 #include "mpfr-test.h"
 
@@ -34,7 +31,7 @@ MA 02110-1301, USA. */
   mpfr_set_str_binary (x, t); \
   if (mpfr_cmp (x, y)) \
     { \
-      printf ("Error in mpfr_set_str (%u):\n", n); \
+      printf ("Error in mpfr_set_str (%d):\n", n); \
       mpfr_print_binary (x); \
       puts (""); \
       mpfr_print_binary (y); \
@@ -48,7 +45,7 @@ static void
 check_underflow (void)
 {
   mpfr_t a;
-  mp_exp_t emin, emax;
+  mpfr_exp_t emin, emax;
   int res;
 
   mpfr_init (a);
@@ -56,10 +53,10 @@ check_underflow (void)
   /* Check underflow */
   emin = mpfr_get_emin ();
   set_emin (-20);
-  res = mpfr_set_str (a, "0.00000000001", 10, GMP_RNDZ);
+  res = mpfr_set_str (a, "0.00000000001", 10, MPFR_RNDZ);
   if (!MPFR_IS_ZERO (a))
     {
-      printf("ERROR for mpfr_set_str (a, \"0.00000000001\", 10, GMP_RNDN)\n"
+      printf("ERROR for mpfr_set_str (a, \"0.00000000001\", 10, MPFR_RNDN)\n"
              " with emin=-20\n"
              "res=%d\n", res);
       mpfr_dump (a);
@@ -70,10 +67,10 @@ check_underflow (void)
   /* check overflow */
   emax = mpfr_get_emax ();
   set_emax (1073741823); /* 2^30-1 */
-  mpfr_set_str (a, "2E1000000000", 10, GMP_RNDN);
+  mpfr_set_str (a, "2E1000000000", 10, MPFR_RNDN);
   if (!mpfr_inf_p (a) || mpfr_sgn (a) < 0)
     {
-      printf("ERROR for mpfr_set_str (a, \"2E1000000000\", 10, GMP_RNDN);\n");
+      printf("ERROR for mpfr_set_str (a, \"2E1000000000\", 10, MPFR_RNDN);\n");
       exit (1);
     }
   set_emax (emax);
@@ -81,24 +78,43 @@ check_underflow (void)
   mpfr_clear (a);
 }
 
+/* Bug found by Christoph Lauter. */
+static void
+bug20081028 (void)
+{
+  mpfr_t x;
+  const char *s = "0.10000000000000000000000000000001E1";
+
+  mpfr_init2 (x, 32);
+  mpfr_set_str (x, "1.00000000000000000006", 10, MPFR_RNDU);
+  if (! mpfr_greater_p (x, __gmpfr_one))
+    {
+      printf ("Error in bug20081028:\nExpected %s\nGot      ", s);
+      mpfr_dump (x);
+      exit (1);
+    }
+  mpfr_clear (x);
+}
+
 int
 main (int argc, char *argv[])
 {
   mpfr_t x, y;
   unsigned long k, bd, nc, i;
   char *str, *str2;
-  mp_exp_t e;
-  int base, logbase, prec, baseprec, ret;
+  mpfr_exp_t e;
+  int base, logbase, prec, baseprec, ret, obase;
 
   tests_start_mpfr ();
 
-  if (argc >= 2) /* tset_str <string> [<prec>] [<base>] */
+  if (argc >= 2) /* tset_str <string> [<prec>] [<ibase>] [<obase>] */
     {
       prec = (argc >= 3) ? atoi (argv[2]) : 53;
       base = (argc >= 4) ? atoi (argv[3]) : 2;
+      obase = (argc >= 5) ? atoi (argv[4]) : 10;
       mpfr_init2 (x, prec);
-      mpfr_set_str (x, argv[1], base, GMP_RNDN);
-      mpfr_out_str (stdout, 10, 0, x, GMP_RNDN);
+      mpfr_set_str (x, argv[1], base, MPFR_RNDN);
+      mpfr_out_str (stdout, obase, 0, x, MPFR_RNDN);
       puts ("");
       mpfr_clear (x);
       return 0;
@@ -112,7 +128,7 @@ main (int argc, char *argv[])
 
   bd = randlimb () & 8;
 
-  str2 = str = (char*) (*__gmp_allocate_func) (nc * sizeof(char));
+  str2 = str = (char*) (*__gmp_allocate_func) (nc);
 
   if (bd)
     {
@@ -136,7 +152,7 @@ main (int argc, char *argv[])
   mpfr_set_prec (x, 54);
   mpfr_set_str_binary (x, "0.100100100110110101001010010101111000001011100100101010E-529");
   mpfr_init2 (y, 54);
-  mpfr_set_str (y, "4.936a52bc17254@-133", 16, GMP_RNDN);
+  mpfr_set_str (y, "4.936a52bc17254@-133", 16, MPFR_RNDN);
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (1a):\n");
@@ -150,7 +166,7 @@ main (int argc, char *argv[])
     }
 
   mpfr_set_str_binary (x, "0.111111101101110010111010100110000111011001010100001101E-529");
-  mpfr_set_str (y, "0.fedcba98765434P-529", 16, GMP_RNDN);
+  mpfr_set_str (y, "0.fedcba98765434P-529", 16, MPFR_RNDN);
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (1b):\n");
@@ -163,7 +179,7 @@ main (int argc, char *argv[])
       exit (1);
     }
 
-  (*__gmp_free_func) (str, nc * sizeof(char));
+  (*__gmp_free_func) (str, nc);
 
   mpfr_set_prec (x, 53);
   mpfr_set_str_binary (x, "+110101100.01010000101101000000100111001000101011101110E00");
@@ -188,11 +204,11 @@ main (int argc, char *argv[])
       exit (1);
     }
 
-  mpfr_set_str (x, "+243495834958.53452345E1", 10, GMP_RNDN);
-  mpfr_set_str (x, "9007199254740993", 10, GMP_RNDN);
-  mpfr_set_str (x, "9007199254740992", 10, GMP_RNDU);
-  mpfr_set_str (x, "9007199254740992", 10, GMP_RNDD);
-  mpfr_set_str (x, "9007199254740992", 10, GMP_RNDZ);
+  mpfr_set_str (x, "+243495834958.53452345E1", 10, MPFR_RNDN);
+  mpfr_set_str (x, "9007199254740993", 10, MPFR_RNDN);
+  mpfr_set_str (x, "9007199254740992", 10, MPFR_RNDU);
+  mpfr_set_str (x, "9007199254740992", 10, MPFR_RNDD);
+  mpfr_set_str (x, "9007199254740992", 10, MPFR_RNDZ);
 
   /* check a random number printed and read is not modified */
   prec = 53;
@@ -200,8 +216,10 @@ main (int argc, char *argv[])
   mpfr_set_prec (y, prec);
   for (i=0;i<N;i++)
     {
-      mpfr_random (x);
-      k = RND_RAND ();
+      mpfr_rnd_t rnd;
+
+      mpfr_urandomb (x, RANDS);
+      rnd = RND_RAND ();
       logbase = (randlimb () % 5) + 1;
       base = 1 << logbase;
       /* Warning: the number of bits needed to print exactly a number of
@@ -212,13 +230,13 @@ main (int argc, char *argv[])
         baseprec = prec;
       else
         baseprec = 1 + (prec - 2 + logbase) / logbase;
-      str = mpfr_get_str (NULL, &e, base, baseprec, x, (mp_rnd_t) k);
-      mpfr_set_str (y, str, base, (mp_rnd_t) k);
+      str = mpfr_get_str (NULL, &e, base, baseprec, x, rnd);
+      mpfr_set_str (y, str, base, rnd);
       MPFR_EXP(y) += logbase * (e - strlen (str));
       if (mpfr_cmp (x, y))
         {
           printf ("mpfr_set_str o mpfr_get_str <> id for rnd_mode=%s\n",
-                  mpfr_print_rnd_mode ((mp_rnd_t) k));
+                  mpfr_print_rnd_mode (rnd));
           printf ("x=");
           mpfr_print_binary (x);
           puts ("");
@@ -233,9 +251,9 @@ main (int argc, char *argv[])
       (*__gmp_free_func) (str, strlen (str) + 1);
     }
 
-  for (i = 2; i <= 36; i++)
+  for (i = 2; i <= 62; i++)
     {
-      if (mpfr_set_str (x, "@NaN@(garbage)", i, GMP_RNDN) != 0 ||
+      if (mpfr_set_str (x, "@NaN@(garbage)", i, MPFR_RNDN) != 0 ||
           !mpfr_nan_p(x))
         {
           printf ("mpfr_set_str failed on @NaN@(garbage)\n");
@@ -243,21 +261,21 @@ main (int argc, char *argv[])
         }
 
       /*
-      if (mpfr_set_str (x, "@Inf@garbage", i, GMP_RNDN) != 0 ||
+      if (mpfr_set_str (x, "@Inf@garbage", i, MPFR_RNDN) != 0 ||
           !mpfr_inf_p(x) || MPFR_SIGN(x) < 0)
         {
           printf ("mpfr_set_str failed on @Inf@garbage\n");
           exit (1);
         }
 
-      if (mpfr_set_str (x, "-@Inf@garbage", i, GMP_RNDN) != 0 ||
+      if (mpfr_set_str (x, "-@Inf@garbage", i, MPFR_RNDN) != 0 ||
           !mpfr_inf_p(x) || MPFR_SIGN(x) > 0)
         {
           printf ("mpfr_set_str failed on -@Inf@garbage\n");
           exit (1);
         }
 
-      if (mpfr_set_str (x, "+@Inf@garbage", i, GMP_RNDN) != 0 ||
+      if (mpfr_set_str (x, "+@Inf@garbage", i, MPFR_RNDN) != 0 ||
           !mpfr_inf_p(x) || MPFR_SIGN(x) < 0)
         {
           printf ("mpfr_set_str failed on +@Inf@garbage\n");
@@ -268,28 +286,28 @@ main (int argc, char *argv[])
       if (i > 16)
         continue;
 
-      if (mpfr_set_str (x, "NaN", i, GMP_RNDN) != 0 ||
+      if (mpfr_set_str (x, "NaN", i, MPFR_RNDN) != 0 ||
           !mpfr_nan_p(x))
         {
           printf ("mpfr_set_str failed on NaN\n");
           exit (1);
         }
 
-      if (mpfr_set_str (x, "Inf", i, GMP_RNDN) != 0 ||
+      if (mpfr_set_str (x, "Inf", i, MPFR_RNDN) != 0 ||
           !mpfr_inf_p(x) || MPFR_SIGN(x) < 0)
         {
           printf ("mpfr_set_str failed on Inf\n");
           exit (1);
         }
 
-      if (mpfr_set_str (x, "-Inf", i, GMP_RNDN) != 0 ||
+      if (mpfr_set_str (x, "-Inf", i, MPFR_RNDN) != 0 ||
           !mpfr_inf_p(x) || MPFR_SIGN(x) > 0)
         {
           printf ("mpfr_set_str failed on -Inf\n");
           exit (1);
         }
 
-      if (mpfr_set_str (x, "+Inf", i, GMP_RNDN) != 0 ||
+      if (mpfr_set_str (x, "+Inf", i, MPFR_RNDN) != 0 ||
           !mpfr_inf_p(x) || MPFR_SIGN(x) < 0)
         {
           printf ("mpfr_set_str failed on +Inf\n");
@@ -299,7 +317,7 @@ main (int argc, char *argv[])
 
   /* check that mpfr_set_str works for uppercase letters too */
   mpfr_set_prec (x, 10);
-  mpfr_set_str (x, "B", 16, GMP_RNDN);
+  mpfr_set_str (x, "B", 16, MPFR_RNDN);
   if (mpfr_cmp_ui (x, 11) != 0)
     {
       printf ("mpfr_set_str does not work for uppercase letters\n");
@@ -312,7 +330,7 @@ main (int argc, char *argv[])
   mpfr_set_prec (x, 64);
   mpfr_set_prec (y, 64);
   mpfr_set_str_binary (x, "1.0E-532");
-  mpfr_set_str (y, "0.71128279983522479470@-160", 10, GMP_RNDU);
+  mpfr_set_str (y, "0.71128279983522479470@-160", 10, MPFR_RNDU);
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_set_str (2):\n");
@@ -332,7 +350,7 @@ main (int argc, char *argv[])
   mpfr_set_prec (x, 64);
   mpfr_set_prec (y, 64);
   mpfr_set_str_binary (x, "1.111111111110000000000000000111111111111111111111111110000000001E184");
-  mpfr_set_str (y, "0.jo08hg31hc5mmpj5mjjmgn55p2h35g@39", 27, GMP_RNDU);
+  mpfr_set_str (y, "0.jo08hg31hc5mmpj5mjjmgn55p2h35g@39", 27, MPFR_RNDU);
   /* y = 49027884868983130654865109690613178467841148597221480052 */
   if (mpfr_cmp (x, y))
     {
@@ -349,14 +367,14 @@ main (int argc, char *argv[])
   /* not exact rounding in mpfr_set_str
      same number with more precision is : 1.111111111111111111111111111000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000011011111101000001101110110010101101000010100110011101110010001110e195
      this result is the same as mpfr_set_str */
-  /* problem was : can_round was call with GMP_RNDN round mode,
+  /* problem was : can_round was call with MPFR_RNDN round mode,
      so can_round use an error : 1/2 * 2^err * ulp(y)
      instead of 2^err * ulp(y)
      I have increase err by 1 */
   mpfr_set_prec (x, 64);  /* it was round down instead of up */
   mpfr_set_prec (y, 64);
   mpfr_set_str_binary (x, "1.111111111111111111111111111000000000000000000000000000000000001e195");
-  mpfr_set_str (y, "0.6e23ekb6acgh96abk10b6c9f2ka16i@45", 21, GMP_RNDU);
+  mpfr_set_str (y, "0.6e23ekb6acgh96abk10b6c9f2ka16i@45", 21, MPFR_RNDU);
   /* y = 100433627392042473064661483711179345482301462325708736552078 */
   if (mpfr_cmp (x, y))
     {
@@ -375,7 +393,7 @@ main (int argc, char *argv[])
   mpfr_set_prec (x, 64);  /* it was round down instead of up */
   mpfr_set_prec (y, 64);
   mpfr_set_str_binary (x, "1.111111100000001111110000000000011111011111111111111111111111111e180");
-  mpfr_set_str (y, "0.10j8j2k82ehahha56390df0a1de030@41", 23, GMP_RNDZ);
+  mpfr_set_str (y, "0.10j8j2k82ehahha56390df0a1de030@41", 23, MPFR_RNDZ);
   /* y = 3053110535624388280648330929253842828159081875986159414 */
   if (mpfr_cmp (x, y))
     {
@@ -391,7 +409,7 @@ main (int argc, char *argv[])
 
   mpfr_set_prec (x, 64);
   mpfr_set_prec (y, 64);
-  mpfr_set_str (y, "0.jrchfhpp9en7hidqm9bmcofid9q3jg@39", 28, GMP_RNDU);
+  mpfr_set_str (y, "0.jrchfhpp9en7hidqm9bmcofid9q3jg@39", 28, MPFR_RNDU);
   /* y = 196159429139499688661464718784226062699788036696626429952 */
   mpfr_set_str_binary (x, "0.1111111111111111111111111111111000000000000011100000001111100001E187");
   if (mpfr_cmp (x, y))
@@ -408,7 +426,7 @@ main (int argc, char *argv[])
 
   mpfr_set_prec (x, 64);
   mpfr_set_prec (y, 64);
-  mpfr_set_str (y, "0.h148m5ld5cf8gk1kd70b6ege92g6ba@47", 24, GMP_RNDZ);
+  mpfr_set_str (y, "0.h148m5ld5cf8gk1kd70b6ege92g6ba@47", 24, MPFR_RNDZ);
   /* y = 52652933527468502324759448399183654588831274530295083078827114496 */
   mpfr_set_str_binary (x, "0.1111111111111100000000001000000000000000000011111111111111101111E215");
   if (mpfr_cmp (x, y))
@@ -427,7 +445,7 @@ main (int argc, char *argv[])
   mpfr_set_prec (x, 53);
   mpfr_set_prec (y, 53);
 
-  mpfr_set_str (y, "5e125", 10, GMP_RNDN);
+  mpfr_set_str (y, "5e125", 10, MPFR_RNDN);
   mpfr_set_str_binary (x, "0.10111101000101110110011000100000101001010000000111111E418");
   if (mpfr_cmp (x, y))
     {
@@ -441,7 +459,7 @@ main (int argc, char *argv[])
       exit (1);
     }
 
-  mpfr_set_str (y, "69e267", 10, GMP_RNDN);
+  mpfr_set_str (y, "69e267", 10, MPFR_RNDN);
   mpfr_set_str_binary (x, "0.10000101101111100101101100000110010011001010011011010E894");
   if (mpfr_cmp (x, y))
     {
@@ -455,7 +473,7 @@ main (int argc, char *argv[])
       exit (1);
     }
 
-  mpfr_set_str (y, "623e100", 10, GMP_RNDN);
+  mpfr_set_str (y, "623e100", 10, MPFR_RNDN);
   mpfr_set_str_binary (x, "0.10110010000001010011000101111001110101000001111011111E342");
   if (mpfr_cmp (x, y))
     {
@@ -469,7 +487,7 @@ main (int argc, char *argv[])
       exit (1);
     }
 
-  mpfr_set_str (y, "3571e263", 10, GMP_RNDN);
+  mpfr_set_str (y, "3571e263", 10, MPFR_RNDN);
   mpfr_set_str_binary (x, "0.10110001001100100010011000110000111010100000110101010E886");
   if (mpfr_cmp (x, y))
     {
@@ -483,7 +501,7 @@ main (int argc, char *argv[])
       exit (1);
     }
 
-  mpfr_set_str (y, "75569e-254", 10, GMP_RNDN);
+  mpfr_set_str (y, "75569e-254", 10, MPFR_RNDN);
   mpfr_set_str_binary (x, "0.10101101001000110001011011001000111000110101010110011E-827");
   if (mpfr_cmp (x, y))
     {
@@ -497,7 +515,7 @@ main (int argc, char *argv[])
       exit (1);
     }
 
-  mpfr_set_str (y, "920657e-23", 10, GMP_RNDN);
+  mpfr_set_str (y, "920657e-23", 10, MPFR_RNDN);
   mpfr_set_str_binary (x, "0.10101001110101001100110000101110110111101111001101100E-56");
   if (mpfr_cmp (x, y))
     {
@@ -511,7 +529,7 @@ main (int argc, char *argv[])
       exit (1);
     }
 
-  mpfr_set_str (y, "9210917e80", 10, GMP_RNDN);
+  mpfr_set_str (y, "9210917e80", 10, MPFR_RNDN);
   mpfr_set_str_binary (x, "0.11101101000100011001000110100011111100110000000110010E289");
   if (mpfr_cmp (x, y))
     {
@@ -525,7 +543,7 @@ main (int argc, char *argv[])
       exit (1);
     }
 
-  mpfr_set_str (y, "87575437e-309", 10, GMP_RNDN);
+  mpfr_set_str (y, "87575437e-309", 10, MPFR_RNDN);
   mpfr_set_str_binary (x, "0.11110000001110011001000000110000000100000010101101100E-1000");
   if (mpfr_cmp (x, y))
     {
@@ -539,7 +557,7 @@ main (int argc, char *argv[])
       exit (1);
     }
 
-  mpfr_set_str (y, "245540327e122", 10, GMP_RNDN);
+  mpfr_set_str (y, "245540327e122", 10, MPFR_RNDN);
   mpfr_set_str_binary (x, "0.10001101101100010001100011110000110001100010111001011E434");
   if (mpfr_cmp (x, y))
     {
@@ -553,7 +571,7 @@ main (int argc, char *argv[])
       exit (1);
     }
 
-  mpfr_set_str (y, "491080654e122", 10, GMP_RNDN);
+  mpfr_set_str (y, "491080654e122", 10, MPFR_RNDN);
   mpfr_set_str_binary (x, "0.10001101101100010001100011110000110001100010111001011E435");
   if (mpfr_cmp (x, y))
     {
@@ -567,7 +585,7 @@ main (int argc, char *argv[])
       exit (1);
     }
 
-  mpfr_set_str (y, "83356057653e193", 10, GMP_RNDN);
+  mpfr_set_str (y, "83356057653e193", 10, MPFR_RNDN);
   mpfr_set_str_binary (x, "0.10101010001001110011011011010111011100010101000011000E678");
   if (mpfr_cmp (x, y))
     {
@@ -581,150 +599,150 @@ main (int argc, char *argv[])
       exit (1);
     }
 
-  CHECK53(y, "83356057653e193", GMP_RNDN, x,
+  CHECK53(y, "83356057653e193", MPFR_RNDN, x,
           "0.10101010001001110011011011010111011100010101000011000E678",
           18);
 
-  CHECK53(y, "619534293513e124", GMP_RNDN, x,
+  CHECK53(y, "619534293513e124", MPFR_RNDN, x,
           "0.10001000011000010000000110000001111111110000011110001e452",
           19);
 
-  CHECK53(y, "3142213164987e-294", GMP_RNDN, x,
+  CHECK53(y, "3142213164987e-294", MPFR_RNDN, x,
           "0.11101001101000000100111011111101111001010001001101111e-935",
           20);
 
-  CHECK53(y, "36167929443327e-159", GMP_RNDN, x,
+  CHECK53(y, "36167929443327e-159", MPFR_RNDN, x,
           "0.11100111001110111110000101011001100110010100011111100e-483",
           21);
 
-  CHECK53(y, "904198236083175e-161", GMP_RNDN, x,
+  CHECK53(y, "904198236083175e-161", MPFR_RNDN, x,
           "0.11100111001110111110000101011001100110010100011111100e-485",
           22);
 
-  CHECK53(y, "3743626360493413e-165", GMP_RNDN, x,
+  CHECK53(y, "3743626360493413e-165", MPFR_RNDN, x,
           "0.11000100000100011101001010111101011011011111011111001e-496",
           23);
 
-  CHECK53(y, "94080055902682397e-242", GMP_RNDN, x,
+  CHECK53(y, "94080055902682397e-242", MPFR_RNDN, x,
           "0.10110010010011000000111100011100111100110011011001010e-747",
           24);
 
-  CHECK53(y, "7e-303", GMP_RNDD, x,
+  CHECK53(y, "7e-303", MPFR_RNDD, x,
           "0.10011001100111001000100110001110001000110111110001011e-1003",
           25);
-  CHECK53(y, "7e-303", GMP_RNDU, x,
+  CHECK53(y, "7e-303", MPFR_RNDU, x,
           "0.10011001100111001000100110001110001000110111110001100e-1003",
           26);
 
-  CHECK53(y, "93e-234", GMP_RNDD, x,
+  CHECK53(y, "93e-234", MPFR_RNDD, x,
           "0.10010011110110010111001001111001000010000000001110101E-770",
           27);
-  CHECK53(y, "93e-234", GMP_RNDU, x,
+  CHECK53(y, "93e-234", MPFR_RNDU, x,
           "0.10010011110110010111001001111001000010000000001110110E-770",
           28);
 
-  CHECK53(y, "755e174", GMP_RNDD, x,
+  CHECK53(y, "755e174", MPFR_RNDD, x,
           "0.10111110110010011000110010011111101111000111111000101E588",
           29);
-  CHECK53(y, "755e174", GMP_RNDU, x,
+  CHECK53(y, "755e174", MPFR_RNDU, x,
           "0.10111110110010011000110010011111101111000111111000110E588",
           30);
 
-  CHECK53(y, "8699e-276", GMP_RNDD, x,
+  CHECK53(y, "8699e-276", MPFR_RNDD, x,
           "0.10010110100101101111100100100011011101100110100101100E-903",
           31);
-  CHECK53(y, "8699e-276", GMP_RNDU, x,
+  CHECK53(y, "8699e-276", MPFR_RNDU, x,
           "0.10010110100101101111100100100011011101100110100101101E-903",
           32);
 
-  CHECK53(y, "82081e41", GMP_RNDD, x,
+  CHECK53(y, "82081e41", MPFR_RNDD, x,
           "0.10111000000010000010111011111001111010100011111001011E153",
           33);
-  CHECK53(y, "82081e41", GMP_RNDU, x,
+  CHECK53(y, "82081e41", MPFR_RNDU, x,
           "0.10111000000010000010111011111001111010100011111001100E153",
           34);
 
-  CHECK53(y, "584169e229", GMP_RNDD, x,
+  CHECK53(y, "584169e229", MPFR_RNDD, x,
           "0.11101011001010111000001011001110111000111100110101010E780",
           35);
-  CHECK53(y, "584169e229", GMP_RNDU, x,
+  CHECK53(y, "584169e229", MPFR_RNDU, x,
           "0.11101011001010111000001011001110111000111100110101011E780",
           36);
 
-  CHECK53(y, "5783893e-128", GMP_RNDD, x,
+  CHECK53(y, "5783893e-128", MPFR_RNDD, x,
           "0.10011000111100000110011110000101100111110011101110100E-402",
           37);
-  CHECK53(y, "5783893e-128", GMP_RNDU, x,
+  CHECK53(y, "5783893e-128", MPFR_RNDU, x,
           "0.10011000111100000110011110000101100111110011101110101E-402",
           38);
 
-  CHECK53(y, "87575437e-310", GMP_RNDD, x,
+  CHECK53(y, "87575437e-310", MPFR_RNDD, x,
           "0.11000000001011100000110011110011010000000010001010110E-1003",
           39);
-  CHECK53(y, "87575437e-310", GMP_RNDU, x,
+  CHECK53(y, "87575437e-310", MPFR_RNDU, x,
           "0.11000000001011100000110011110011010000000010001010111E-1003",
           40);
 
-  CHECK53(y, "245540327e121", GMP_RNDD, x,
+  CHECK53(y, "245540327e121", MPFR_RNDD, x,
           "0.11100010101101001111010010110100011100000100101000100E430",
           41);
-  CHECK53(y, "245540327e121", GMP_RNDU, x,
+  CHECK53(y, "245540327e121", MPFR_RNDU, x,
           "0.11100010101101001111010010110100011100000100101000101E430",
           42);
 
-  CHECK53(y, "9078555839e-109", GMP_RNDD, x,
+  CHECK53(y, "9078555839e-109", MPFR_RNDD, x,
           "0.11111110001010111010110000110011100110001010011101101E-329",
           43);
-  CHECK53(y, "9078555839e-109", GMP_RNDU, x,
+  CHECK53(y, "9078555839e-109", MPFR_RNDU, x,
           "0.11111110001010111010110000110011100110001010011101110E-329",
           44);
 
-  CHECK53(y, "42333842451e201", GMP_RNDD, x,
+  CHECK53(y, "42333842451e201", MPFR_RNDD, x,
           "0.10000000110001001101000100110110111110101011101011111E704",
           45);
-  CHECK53(y, "42333842451e201", GMP_RNDU, x,
+  CHECK53(y, "42333842451e201", MPFR_RNDU, x,
           "0.10000000110001001101000100110110111110101011101100000E704",
           46);
 
-  CHECK53(y, "778380362293e218", GMP_RNDD, x,
+  CHECK53(y, "778380362293e218", MPFR_RNDD, x,
           "0.11001101010111000001001100001100110010000001010010010E764",
           47);
-  CHECK53(y, "778380362293e218", GMP_RNDU, x,
+  CHECK53(y, "778380362293e218", MPFR_RNDU, x,
           "0.11001101010111000001001100001100110010000001010010011E764",
           48);
 
-  CHECK53(y, "7812878489261e-179", GMP_RNDD, x,
+  CHECK53(y, "7812878489261e-179", MPFR_RNDD, x,
           "0.10010011011011010111001111011101111101101101001110100E-551",
           49);
-  CHECK53(y, "7812878489261e-179", GMP_RNDU, x,
+  CHECK53(y, "7812878489261e-179", MPFR_RNDU, x,
           "0.10010011011011010111001111011101111101101101001110101E-551",
           50);
 
-  CHECK53(y, "77003665618895e-73", GMP_RNDD, x,
+  CHECK53(y, "77003665618895e-73", MPFR_RNDD, x,
           "0.11000101111110111111001111111101001101111000000101001E-196",
           51);
-  CHECK53(y, "77003665618895e-73", GMP_RNDU, x,
+  CHECK53(y, "77003665618895e-73", MPFR_RNDU, x,
           "0.11000101111110111111001111111101001101111000000101010E-196",
           52);
 
-  CHECK53(y, "834735494917063e-300", GMP_RNDD, x,
+  CHECK53(y, "834735494917063e-300", MPFR_RNDD, x,
           "0.11111110001101100001001101111100010011001110111010001E-947",
           53);
-  CHECK53(y, "834735494917063e-300", GMP_RNDU, x,
+  CHECK53(y, "834735494917063e-300", MPFR_RNDU, x,
           "0.11111110001101100001001101111100010011001110111010010E-947",
           54);
 
-  CHECK53(y, "6182410494241627e-119", GMP_RNDD, x,
+  CHECK53(y, "6182410494241627e-119", MPFR_RNDD, x,
           "0.10001101110010110010001011000010001000101110100000111E-342",
           55);
-  CHECK53(y, "6182410494241627e-119", GMP_RNDU, x,
+  CHECK53(y, "6182410494241627e-119", MPFR_RNDU, x,
           "0.10001101110010110010001011000010001000101110100001000E-342",
           56);
 
-  CHECK53(y, "26153245263757307e49", GMP_RNDD, x,
+  CHECK53(y, "26153245263757307e49", MPFR_RNDD, x,
           "0.10011110111100000000001011011110101100010000011011110E218",
           57);
-  CHECK53(y, "26153245263757307e49", GMP_RNDU, x,
+  CHECK53(y, "26153245263757307e49", MPFR_RNDU, x,
           "0.10011110111100000000001011011110101100010000011011111E218",
           58);
 
@@ -737,7 +755,7 @@ main (int argc, char *argv[])
     size_t nb_digit = mp_bits_per_limb;
     mp_limb_t check_limb[2] = {MPFR_LIMB_HIGHBIT, ~(MPFR_LIMB_HIGHBIT >> 1)};
     int base[3] = {10, 16, 19};
-    mp_rnd_t rnd[3] = {GMP_RNDU, GMP_RNDN, GMP_RNDD};
+    mpfr_rnd_t rnd[3] = {MPFR_RNDU, MPFR_RNDN, MPFR_RNDD};
     int cbase, climb, crnd;
     char *str;
 
@@ -746,14 +764,14 @@ main (int argc, char *argv[])
 
     str = (char*) (*__gmp_allocate_func) (N + 20);
 
-    mpfr_set_ui (x, 1, GMP_RNDN); /* ensures that x is not NaN or Inf */
+    mpfr_set_ui (x, 1, MPFR_RNDN); /* ensures that x is not NaN or Inf */
     for (; nb_digit < N; nb_digit *= 10)
       for (cbase = 0; cbase < 3; cbase++)
         for (climb = 0; climb < 2; climb++)
           for (crnd = 0; crnd < 3; crnd++)
             {
               char *str1;
-              mp_exp_t exp;
+              mpfr_exp_t exp;
 
               *(MPFR_MANT(x)) = check_limb[climb];
               MPFR_EXP(x) = 0;
@@ -792,13 +810,13 @@ main (int argc, char *argv[])
 
   /* check that flags are correctly cleared */
   mpfr_set_nan (x);
-  mpfr_set_str (x, "+0.0", 10, GMP_RNDN);
+  mpfr_set_str (x, "+0.0", 10, MPFR_RNDN);
   if (!mpfr_number_p(x) || mpfr_cmp_ui (x, 0) != 0 || mpfr_sgn (x) < 0)
     {
       printf ("x <- +0.0 failed after x=NaN\n");
       exit (1);
     }
-  mpfr_set_str (x, "-0.0", 10, GMP_RNDN);
+  mpfr_set_str (x, "-0.0", 10, MPFR_RNDN);
   if (!mpfr_number_p(x) || mpfr_cmp_ui (x, 0) != 0 || mpfr_sgn (x) > 0)
     {
       printf ("x <- -0.0 failed after x=NaN\n");
@@ -806,24 +824,24 @@ main (int argc, char *argv[])
     }
 
   /* check invalid input */
-  ret = mpfr_set_str (x, "1E10toto", 10, GMP_RNDN);
+  ret = mpfr_set_str (x, "1E10toto", 10, MPFR_RNDN);
   MPFR_ASSERTN (ret == -1);
-  ret = mpfr_set_str (x, "1p10toto", 16, GMP_RNDN);
+  ret = mpfr_set_str (x, "1p10toto", 16, MPFR_RNDN);
   MPFR_ASSERTN (ret == -1);
-  ret = mpfr_set_str (x, "", 16, GMP_RNDN);
+  ret = mpfr_set_str (x, "", 16, MPFR_RNDN);
   MPFR_ASSERTN (ret == -1);
-  ret = mpfr_set_str (x, "+", 16, GMP_RNDN);
+  ret = mpfr_set_str (x, "+", 16, MPFR_RNDN);
   MPFR_ASSERTN (ret == -1);
-  ret = mpfr_set_str (x, "-", 16, GMP_RNDN);
+  ret = mpfr_set_str (x, "-", 16, MPFR_RNDN);
   MPFR_ASSERTN (ret == -1);
-  ret = mpfr_set_str (x, "this_is_an_invalid_number_in_base_36", 36, GMP_RNDN);
+  ret = mpfr_set_str (x, "this_is_an_invalid_number_in_base_36", 36, MPFR_RNDN);
   MPFR_ASSERTN (ret == -1);
-  ret = mpfr_set_str (x, "1.2.3", 10, GMP_RNDN);
+  ret = mpfr_set_str (x, "1.2.3", 10, MPFR_RNDN);
   MPFR_ASSERTN (ret == -1);
   mpfr_set_prec (x, 135);
-  ret = mpfr_set_str (x, "thisisavalidnumberinbase36", 36, GMP_RNDN);
+  ret = mpfr_set_str (x, "thisisavalidnumberinbase36", 36, MPFR_RNDN);
   mpfr_set_prec (y, 135);
-  mpfr_set_str (y, "23833565676460972739462619524519814462546", 10, GMP_RNDN);
+  mpfr_set_str (y, "23833565676460972739462619524519814462546", 10, MPFR_RNDN);
   MPFR_ASSERTN (mpfr_cmp (x, y) == 0 && ret == 0);
 
   /* coverage test for set_str_binary */
@@ -845,6 +863,7 @@ main (int argc, char *argv[])
   mpfr_clear (y);
 
   check_underflow ();
+  bug20081028 ();
 
   tests_end_mpfr ();
   return 0;
